# Generated by Django 2.2 on 2022-07-21 20:45

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wcd_jwt_registry', '0001_initial'),
        ('wcd_device_recognizer', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TokenInterlocutorConnection',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('interlocutor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='token_connections', to='wcd_device_recognizer.Interlocutor', verbose_name='Connected interlocutor')),
                ('network', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='token_connections', to='wcd_device_recognizer.InterlocutorNetwork', verbose_name='Connected interlocutor network')),
                ('token', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='interlocutor_connections', to='wcd_jwt_registry.Token', verbose_name='Connected token')),
            ],
            options={
                'verbose_name': 'Interlocutor token connection',
                'verbose_name_plural': 'Interlocutor token connections',
            },
        ),
    ]
