# Generated by Django 2.2 on 2022-07-22 07:27

import datetime
from django.db import migrations, models
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('wcd_jwt_registry', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='token',
            name='internal_expired_at',
            field=models.DateTimeField(default=datetime.datetime(2022, 7, 23, 7, 27, 51, 800647, tzinfo=utc), editable=False, verbose_name='Expired at'),
            preserve_default=False,
        ),
    ]
